==> xlr-server-24.3.10-727.913-sources.jar: com/xebialabs/xlrelease/server/jetty/ReleaseJettyServletWebServerFactory.scala <==
package com.xebialabs.xlrelease.server.jetty

import com.xebialabs.deployit.ServerConfiguration
import com.xebialabs.deployit.plumbing.CSPFilter.POLICY_DIRECTIVES_PARAM
import com.xebialabs.deployit.plumbing._
import com.xebialabs.deployit.security.PermissionEnforcer
import com.xebialabs.xlplatform.endpoints.servlet.{PekkoStreamServlet, PekkoStreamServletInitializer}
import com.xebialabs.xlrelease.Environment
import com.xebialabs.xlrelease.config.XlrConfig
import com.xebialabs.xlrelease.security.HttpSessionListenerWithTimeout
import io.micrometer.core.instrument.{MeterRegistry, Tags}
import io.micrometer.jetty12.server.TimedHandler
import org.eclipse.jetty.ee10.servlet.security.ConstraintSecurityHandler
import org.eclipse.jetty.ee10.servlet.{ErrorHandler, ErrorPageErrorHandler, FilterHolder, ServletHolder, SessionHandler}
import org.eclipse.jetty.ee10.servlets.CrossOriginFilter
import org.eclipse.jetty.ee10.webapp.{AbstractConfiguration, Configuration, WebAppContext}
import org.eclipse.jetty.http.HttpCookie.SameSite
import org.eclipse.jetty.server.handler.gzip.GzipHandler
import org.eclipse.jetty.util.compression.{CompressionPool, DeflaterPool}
import org.springframework.boot.web.embedded.jetty.JettyServletWebServerFactory
import org.springframework.boot.web.server.ErrorPage
import org.springframework.security.web.context.AbstractSecurityWebApplicationInitializer
import org.springframework.web.filter.{CharacterEncodingFilter, DelegatingFilterProxy}

import java.nio.file.{LinkOption, Path}
import java.util
import jakarta.servlet.{DispatcherType, SessionTrackingMode}
import scala.jdk.CollectionConverters._

class ReleaseJettyServletWebServerFactory(val serverConfiguration: ServerConfiguration,
                                          val xlrConfig: XlrConfig,
                                          val permissionEnforcer: PermissionEnforcer,
                                          meterRegistry: MeterRegistry)
  extends JettyServletWebServerFactory {

  override def isRegisterDefaultServlet = false

  override protected def postProcessWebAppContext(contextRoot: WebAppContext): Unit = {
    setupJetty(contextRoot)
    setupLogbackFilter(contextRoot)
    setupSpring(contextRoot)
    setupCorsFilter(contextRoot)
    setupXssSecurityHeadersFilter(contextRoot)
    setupUrlFilter(contextRoot)
    setupCSPFilter(contextRoot)
    setupSlowdownFilter(contextRoot)
    setupExtensionApi(contextRoot)
    setupVersionEnforcer(contextRoot)
    setupRequestLocal(contextRoot)
    setupMaintenanceModeAccessControlFilter(contextRoot)
    setupSseDisableBufferingFilter(contextRoot)
  }

  private def setupJetty(contextRoot: WebAppContext): Unit = {
    contextRoot.setSessionHandler(newSessionHandler)
    contextRoot.setSecurityHandler(new ConstraintSecurityHandler)
    contextRoot.addConfiguration(getErrorPageConfiguration())
    if (xlrConfig.metrics.enabled) {
      enableInstrumentedHandler(contextRoot)
    }
    if (xlrConfig.server_http_gzip_enabled) {
      enableGzipHandler(contextRoot)
    }
    if (xlrConfig.server_http_cache_enabled && Environment.isProduction) {
      enableResourcesCache(contextRoot)
    }
  }

  private def getErrorPageConfiguration(): Configuration = {
    // replaces org.springframework.boot.web.embedded.jetty.JettyServletWebServerFactory.getErrorPageConfiguration
    new AbstractConfiguration(new AbstractConfiguration.Builder()) {
      override def configure(context: WebAppContext): Unit = {
        val errorHandler = new JettyEmbeddedErrorHandler()
        errorHandler.setShowServlet(false)
        errorHandler.setShowStacks(false)
        errorHandler.setShowMessageInTitle(false)
        context.setErrorHandler(errorHandler)
        addJettyErrorPages(errorHandler, getErrorPages())
      }
    }
  }

  private def addJettyErrorPages(errorHandler: ErrorHandler, errorPages: util.Collection[ErrorPage]): Unit = {
    // taken from org.springframework.boot.web.embedded.jetty.JettyServletWebServerFactory.addJettyErrorPages
    import scala.jdk.CollectionConverters._
    errorHandler match {
      case handler: ErrorPageErrorHandler =>
        for (errorPage <- errorPages.asScala) {
          if (errorPage.isGlobal) {
            handler.addErrorPage(ErrorPageErrorHandler.GLOBAL_ERROR_PAGE, errorPage.getPath)
          } else if (errorPage.getExceptionName != null) {
            handler.addErrorPage(errorPage.getExceptionName, errorPage.getPath)
          } else {
            handler.addErrorPage(errorPage.getStatusCode, errorPage.getPath)
          }
        }
      case _ => ()
    }
  }

  private def setupLogbackFilter(contextRoot: WebAppContext): Unit = {
    contextRoot.addFilter(classOf[LogbackFilter], "/*", util.EnumSet.of(DispatcherType.REQUEST)).setAsyncSupported(true)
  }

  private def setupSpring(contextRoot: WebAppContext): Unit = {
    //Set system properties Spring Security uses (via the JRE) for LDAPS
    setPropertyIfNull(SSLConstants.KEYSTORE_PROPERTY, serverConfiguration.getKeyStorePath)
    setPropertyIfNull(SSLConstants.KEYSTORE_PASSWORD_PROPERTY, serverConfiguration.getKeyStorePassword)
    setPropertyIfNull(SSLConstants.TRUSTSTORE_PROPERTY, serverConfiguration.getTrustStorePath)
    setPropertyIfNull(SSLConstants.TRUSTSTORE_PASSWORD_PROPERTY, serverConfiguration.getTrustStorePassword)

    contextRoot.addFilter(classOf[UrlRedirectRewriteFilter], "/*", util.EnumSet.of(DispatcherType.REQUEST))

    val filter = new FilterHolder(classOf[DelegatingFilterProxy])
    filter.setName(AbstractSecurityWebApplicationInitializer.DEFAULT_FILTER_NAME)
    filter.setAsyncSupported(true)
    contextRoot.addFilter(filter, "/*", util.EnumSet.of(DispatcherType.REQUEST))

    val encodingFilter = new FilterHolder(classOf[CharacterEncodingFilter])
    encodingFilter.setName("encodingFilter")
    encodingFilter.setInitParameter("encoding", "UTF-8")
    encodingFilter.setInitParameter("forceEncoding", "true")
    encodingFilter.setAsyncSupported(true)
    contextRoot.addFilter(encodingFilter, "/*", util.EnumSet.of(DispatcherType.REQUEST))

    val resourceFactory = contextRoot.getResourceFactory
    val path = Path.of(".").toRealPath(LinkOption.NOFOLLOW_LINKS).toUri
    val baseResource = resourceFactory.newResource(path)
    contextRoot.setBaseResource(baseResource)

    if (!xlrConfig.server_session_storage_enabled) {
      contextRoot.addEventListener(new HttpSessionListenerWithTimeout)
    }
  }

  private def setupCorsFilter(contextRoot: WebAppContext): Unit = {
    if (xlrConfig.api_corsEnabled) {
      // TODO: Move this to spring security CORS_FILTER
      val holder = new FilterHolder(new CrossOriginFilter)
      holder.setInitParameter(CrossOriginFilter.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
      holder.setInitParameter(CrossOriginFilter.ACCESS_CONTROL_ALLOW_METHODS_HEADER, "POST, GET, OPTIONS, DELETE, PUT")
      holder.setInitParameter(CrossOriginFilter.ACCESS_CONTROL_ALLOW_HEADERS_HEADER, "x-requested-with, accept, origin, content-type")
      holder.setInitParameter(CrossOriginFilter.ACCESS_CONTROL_MAX_AGE_HEADER, "3600")
      holder.setAsyncSupported(true)
      contextRoot.addFilter(holder, "/api/v1/*", util.EnumSet.of(DispatcherType.REQUEST))
    }
  }

  private def setupXssSecurityHeadersFilter(contextRoot: WebAppContext): Unit = {
    val holder = new FilterHolder(new XssSecurityHeadersFilter)
    holder.setAsyncSupported(true)
    contextRoot.addFilter(holder, "/*", util.EnumSet.of(DispatcherType.REQUEST))
  }

  private def setupUrlFilter(contextRoot: WebAppContext): Unit = {
    val holder = new FilterHolder(new IgnoreUrlFilter)
    contextRoot.addFilter(holder, "/*", util.EnumSet.of(DispatcherType.REQUEST))
  }

  private def setupSlowdownFilter(contextRoot: WebAppContext): Unit = {
    if (Environment.isDevelopment) {
      val slowDownInMs = xlrConfig.development_restSlowDownDelay.toMillis
      if (slowDownInMs > 0) {
        logger.warn(s"Configuring slow down filter with $slowDownInMs milliseconds.")
        val slowdownFilter = new FilterHolder(classOf[SlowdownFilter])
        slowdownFilter.setInitParameter(SlowdownFilter.MS, String.valueOf(slowDownInMs))
        slowdownFilter.setAsyncSupported(true)
        contextRoot.addFilter(slowdownFilter, "/*", util.EnumSet.of(DispatcherType.REQUEST))
      }
    }
  }

  private def setupCSPFilter(contextRoot: WebAppContext): Unit = {
    if (xlrConfig.server_http_csp_enabled) {
      val holder = new FilterHolder(new CSPFilter)
      holder.setAsyncSupported(true)
      holder.setInitParameter(POLICY_DIRECTIVES_PARAM, xlrConfig.server_http_csp_policyDirectives)
      contextRoot.addFilter(holder, "/*", util.EnumSet.of(DispatcherType.REQUEST))
    }
  }

  private def setupExtensionApi(contextRoot: WebAppContext): Unit = {
    val apiExtensionRootPath = xlrConfig.serverExtension_rootPath
    val apiExtensionContext = String.format("%s/*", apiExtensionRootPath)
    val extensionApiServletHolder = new ServletHolder(classOf[PekkoStreamServlet])
    extensionApiServletHolder.setDisplayName("ExtensionApiConnectorServlet")
    extensionApiServletHolder.setAsyncSupported(true)
    contextRoot.addEventListener(new PekkoStreamServletInitializer())
    contextRoot.addServlet(extensionApiServletHolder, apiExtensionContext)
  }

  private def setupVersionEnforcer(contextRoot: WebAppContext): Unit = {
    if (!Environment.isDevelopment) {
      contextRoot.addFilter(classOf[VersionEnforcerFilter], "/*", util.EnumSet.of(DispatcherType.REQUEST)).setAsyncSupported(true)
    }
  }

  private def setupRequestLocal(contextRoot: WebAppContext): Unit = {
    contextRoot.addFilter(classOf[RequestLocal], "*", util.EnumSet.of(DispatcherType.REQUEST)).setAsyncSupported(true)
  }

  private def enableResourcesCache(contextRoot: WebAppContext): Unit = {
    val holder = new FilterHolder(new ModifiedResourceFilter)
    holder.setInitParameter("startTimestamp", String.valueOf(System.currentTimeMillis))
    holder.setAsyncSupported(true)
    contextRoot.addFilter(holder, "/static/*", util.EnumSet.of(DispatcherType.REQUEST))
    contextRoot.addFilter(holder, "/ui-extensions/xlrelease-module.js", util.EnumSet.of(DispatcherType.REQUEST))
    contextRoot.addFilter(holder, "/ui-extensions/xlrelease-plugins.js", util.EnumSet.of(DispatcherType.REQUEST))
  }

  private def enableGzipHandler(contextRoot: WebAppContext): Unit = {
    val gzipHandler = new GzipHandler
    gzipHandler.setSyncFlush(true)
    gzipHandler.setIncludedMethods("GET", "POST", "PUT")
    gzipHandler.setMinGzipSize(xlrConfig.server_http_gzip_minSize.intValue)
    gzipHandler.setExcludedPaths(xlrConfig.server_http_gzip_excludedPaths.asScala.toSeq: _*)
    gzipHandler.setDeflaterPool(new DeflaterPool(CompressionPool.DEFAULT_CAPACITY, xlrConfig.server_http_gzip_compression, true))
    contextRoot.insertHandler(gzipHandler)
  }

  private def newSessionHandler = {
    val sessionHandler = new SessionHandler
    sessionHandler.setSessionTrackingModes(util.EnumSet.of(SessionTrackingMode.COOKIE))
    sessionHandler.setHttpOnly(true)
    if (!serverConfiguration.isSsl) {
      logger.debug(s"Setting up Secure Cookie Enabled to - ${serverConfiguration.isSecureCookieEnabled}")
      sessionHandler.getSessionCookieConfig.setSecure(serverConfiguration.isSecureCookieEnabled)
    }
    sessionHandler.setSameSite(SameSite.valueOf(xlrConfig.server_http_cookie_sameSite.getAttributeValue.toUpperCase))
    sessionHandler
  }

  private def enableInstrumentedHandler(contextRoot: WebAppContext): Unit = {
    val handler = new TimedHandler(meterRegistry, Tags.empty())
    contextRoot.insertHandler(handler)
  }

  private def setPropertyIfNull(key: String, value: String): Unit = {
    // skip null new values
    if (value != null) {
      if (System.getProperty(key) == null) {
        if (logger.isDebugEnabled) {
          logger.debug(s"Setting system property - $key")
        }
        System.setProperty(key, value)
      }
      else if (logger.isDebugEnabled) {
        logger.debug(s"Not overriding already specified system property - $key")
      }
    }
  }

  private def setupMaintenanceModeAccessControlFilter(contextRoot: WebAppContext): Unit = {
    if (xlrConfig.maintenanceModeEnabled) {
      val restrictApiAccess = xlrConfig.maintenanceModeRestrictApiAccess
      val filterHolder = new FilterHolder(new MaintenanceModeAccessControlFilter(permissionEnforcer, restrictApiAccess))
      contextRoot.addFilter(filterHolder, "/*", util.EnumSet.of(DispatcherType.REQUEST))
    }
  }
  private def setupSseDisableBufferingFilter(contextRoot: WebAppContext): Unit = {
    val filterHolder = new FilterHolder(new SseDisableBufferingFilter())
    contextRoot.addFilter(filterHolder, "/*", util.EnumSet.of(DispatcherType.REQUEST))
  }


}

==> xlr-server-25.1.0-124.113-sources.jar: com/xebialabs/xlrelease/server/jetty/ReleaseJettyServletWebServerFactory.scala <==
package com.xebialabs.xlrelease.server.jetty

import com.xebialabs.deployit.ServerConfiguration
import com.xebialabs.deployit.plumbing.CSPFilter.POLICY_DIRECTIVES_PARAM
import com.xebialabs.deployit.plumbing._
import com.xebialabs.deployit.security.PermissionEnforcer
import com.xebialabs.xlplatform.endpoints.servlet.{PekkoStreamServlet, PekkoStreamServletInitializer}
import com.xebialabs.xlrelease.Environment
import com.xebialabs.xlrelease.config.XlrConfig
import com.xebialabs.xlrelease.security.HttpSessionListenerWithTimeout
import com.xebialabs.xlrelease.service.XlrServiceManager
import io.micrometer.core.instrument.{MeterRegistry, Tags}
import io.micrometer.jetty12.server.TimedHandler
import org.eclipse.jetty.ee10.servlet.security.ConstraintSecurityHandler
import org.eclipse.jetty.ee10.servlet.{ErrorHandler, ErrorPageErrorHandler, FilterHolder, ServletHolder, SessionHandler}
import org.eclipse.jetty.ee10.servlets.CrossOriginFilter
import org.eclipse.jetty.ee10.webapp.{AbstractConfiguration, Configuration, WebAppContext}
import org.eclipse.jetty.http.HttpCookie.SameSite
import org.eclipse.jetty.server.handler.gzip.GzipHandler
import org.eclipse.jetty.util.compression.{CompressionPool, DeflaterPool}
import org.springframework.boot.web.embedded.jetty.JettyServletWebServerFactory
import org.springframework.boot.web.server.ErrorPage
import org.springframework.security.web.context.AbstractSecurityWebApplicationInitializer
import org.springframework.web.filter.{CharacterEncodingFilter, DelegatingFilterProxy}

import java.nio.file.{LinkOption, Path}
import java.util
import jakarta.servlet.{DispatcherType, SessionTrackingMode}
import scala.jdk.CollectionConverters._

class ReleaseJettyServletWebServerFactory(val serverConfiguration: ServerConfiguration,
                                          val xlrConfig: XlrConfig,
                                          val permissionEnforcer: PermissionEnforcer,
                                          meterRegistry: MeterRegistry,
                                          serviceManager: XlrServiceManager)
  extends JettyServletWebServerFactory {

  override def isRegisterDefaultServlet = false

  override protected def postProcessWebAppContext(contextRoot: WebAppContext): Unit = {
    setupJetty(contextRoot)
    setupLogbackFilter(contextRoot)
    setupSpring(contextRoot)
    setupCorsFilter(contextRoot)
    setupXssSecurityHeadersFilter(contextRoot)
    setupUrlFilter(contextRoot)
    setupCSPFilter(contextRoot)
    setupSlowdownFilter(contextRoot)
    setupExtensionApi(contextRoot)
    setupVersionEnforcer(contextRoot)
    setupRequestLocal(contextRoot)
    setupAccessControlFilter(contextRoot)
  }

  private def setupJetty(contextRoot: WebAppContext): Unit = {
    contextRoot.setSessionHandler(newSessionHandler)
    contextRoot.setSecurityHandler(new ConstraintSecurityHandler)
    contextRoot.addConfiguration(getErrorPageConfiguration())
    if (xlrConfig.metrics.enabled) {
      enableInstrumentedHandler(contextRoot)
    }
    if (xlrConfig.server_http_gzip_enabled) {
      enableGzipHandler(contextRoot)
    }
    if (xlrConfig.server_http_cache_enabled && Environment.isProduction) {
      enableResourcesCache(contextRoot)
    }
  }

  private def getErrorPageConfiguration(): Configuration = {
    // replaces org.springframework.boot.web.embedded.jetty.JettyServletWebServerFactory.getErrorPageConfiguration
    new AbstractConfiguration(new AbstractConfiguration.Builder()) {
      override def configure(context: WebAppContext): Unit = {
        val errorHandler = new JettyEmbeddedErrorHandler()
        errorHandler.setShowServlet(false)
        errorHandler.setShowStacks(false)
        errorHandler.setShowMessageInTitle(false)
        context.setErrorHandler(errorHandler)
        addJettyErrorPages(errorHandler, getErrorPages())
      }
    }
  }

  private def addJettyErrorPages(errorHandler: ErrorHandler, errorPages: util.Collection[ErrorPage]): Unit = {
    // taken from org.springframework.boot.web.embedded.jetty.JettyServletWebServerFactory.addJettyErrorPages
    import scala.jdk.CollectionConverters._
    errorHandler match {
      case handler: ErrorPageErrorHandler =>
        for (errorPage <- errorPages.asScala) {
          if (errorPage.isGlobal) {
            handler.addErrorPage(ErrorPageErrorHandler.GLOBAL_ERROR_PAGE, errorPage.getPath)
          } else if (errorPage.getExceptionName != null) {
            handler.addErrorPage(errorPage.getExceptionName, errorPage.getPath)
          } else {
            handler.addErrorPage(errorPage.getStatusCode, errorPage.getPath)
          }
        }
      case _ => ()
    }
  }

  private def setupLogbackFilter(contextRoot: WebAppContext): Unit = {
    contextRoot.addFilter(classOf[LogbackFilter], "/*", util.EnumSet.of(DispatcherType.REQUEST)).setAsyncSupported(true)
  }

  private def setupSpring(contextRoot: WebAppContext): Unit = {
    //Set system properties Spring Security uses (via the JRE) for LDAPS
    setPropertyIfNull(SSLConstants.KEYSTORE_PROPERTY, serverConfiguration.getKeyStorePath)
    setPropertyIfNull(SSLConstants.KEYSTORE_PASSWORD_PROPERTY, serverConfiguration.getKeyStorePassword)
    setPropertyIfNull(SSLConstants.TRUSTSTORE_PROPERTY, serverConfiguration.getTrustStorePath)
    setPropertyIfNull(SSLConstants.TRUSTSTORE_PASSWORD_PROPERTY, serverConfiguration.getTrustStorePassword)

    contextRoot.addFilter(classOf[UrlRedirectRewriteFilter], "/*", util.EnumSet.of(DispatcherType.REQUEST))

    val filter = new FilterHolder(classOf[DelegatingFilterProxy])
    filter.setName(AbstractSecurityWebApplicationInitializer.DEFAULT_FILTER_NAME)
    filter.setAsyncSupported(true)
    contextRoot.addFilter(filter, "/*", util.EnumSet.of(DispatcherType.REQUEST))

    val encodingFilter = new FilterHolder(classOf[CharacterEncodingFilter])
    encodingFilter.setName("encodingFilter")
    encodingFilter.setInitParameter("encoding", "UTF-8")
    encodingFilter.setInitParameter("forceEncoding", "true")
    encodingFilter.setAsyncSupported(true)
    contextRoot.addFilter(encodingFilter, "/*", util.EnumSet.of(DispatcherType.REQUEST))

    val resourceFactory = contextRoot.getResourceFactory
    val path = Path.of(".").toRealPath(LinkOption.NOFOLLOW_LINKS).toUri
    val baseResource = resourceFactory.newResource(path)
    contextRoot.setBaseResource(baseResource)

    if (!xlrConfig.server_session_storage_enabled) {
      contextRoot.addEventListener(new HttpSessionListenerWithTimeout)
    }
  }

  private def setupCorsFilter(contextRoot: WebAppContext): Unit = {
    if (xlrConfig.api_corsEnabled) {
      // TODO: Move this to spring security CORS_FILTER
      val holder = new FilterHolder(new CrossOriginFilter)
      holder.setInitParameter(CrossOriginFilter.ACCESS_CONTROL_ALLOW_ORIGIN_HEADER, "*")
      holder.setInitParameter(CrossOriginFilter.ACCESS_CONTROL_ALLOW_METHODS_HEADER, "POST, GET, OPTIONS, DELETE, PUT")
      holder.setInitParameter(CrossOriginFilter.ACCESS_CONTROL_ALLOW_HEADERS_HEADER, "x-requested-with, accept, origin, content-type")
      holder.setInitParameter(CrossOriginFilter.ACCESS_CONTROL_MAX_AGE_HEADER, "3600")
      holder.setAsyncSupported(true)
      contextRoot.addFilter(holder, "/api/v1/*", util.EnumSet.of(DispatcherType.REQUEST))
    }
  }

  private def setupXssSecurityHeadersFilter(contextRoot: WebAppContext): Unit = {
    val holder = new FilterHolder(new XssSecurityHeadersFilter)
    holder.setAsyncSupported(true)
    contextRoot.addFilter(holder, "/*", util.EnumSet.of(DispatcherType.REQUEST))
  }

  private def setupUrlFilter(contextRoot: WebAppContext): Unit = {
    val holder = new FilterHolder(new IgnoreUrlFilter)
    contextRoot.addFilter(holder, "/*", util.EnumSet.of(DispatcherType.REQUEST))
  }

  private def setupSlowdownFilter(contextRoot: WebAppContext): Unit = {
    if (Environment.isDevelopment) {
      val slowDownInMs = xlrConfig.development_restSlowDownDelay.toMillis
      if (slowDownInMs > 0) {
        logger.warn(s"Configuring slow down filter with $slowDownInMs milliseconds.")
        val slowdownFilter = new FilterHolder(classOf[SlowdownFilter])
        slowdownFilter.setInitParameter(SlowdownFilter.MS, String.valueOf(slowDownInMs))
        slowdownFilter.setAsyncSupported(true)
        contextRoot.addFilter(slowdownFilter, "/*", util.EnumSet.of(DispatcherType.REQUEST))
      }
    }
  }

  private def setupCSPFilter(contextRoot: WebAppContext): Unit = {
    if (xlrConfig.server_http_csp_enabled) {
      val holder = new FilterHolder(new CSPFilter)
      holder.setAsyncSupported(true)
      holder.setInitParameter(POLICY_DIRECTIVES_PARAM, xlrConfig.server_http_csp_policyDirectives)
      contextRoot.addFilter(holder, "/*", util.EnumSet.of(DispatcherType.REQUEST))
    }
  }

  private def setupExtensionApi(contextRoot: WebAppContext): Unit = {
    val apiExtensionRootPath = xlrConfig.serverExtension_rootPath
    val apiExtensionContext = String.format("%s/*", apiExtensionRootPath)
    val extensionApiServletHolder = new ServletHolder(classOf[PekkoStreamServlet])
    extensionApiServletHolder.setDisplayName("ExtensionApiConnectorServlet")
    extensionApiServletHolder.setAsyncSupported(true)
    contextRoot.addEventListener(new PekkoStreamServletInitializer(xlrConfig.extensionServletSystem.config))
    contextRoot.addServlet(extensionApiServletHolder, apiExtensionContext)
  }

  private def setupVersionEnforcer(contextRoot: WebAppContext): Unit = {
    if (!Environment.isDevelopment) {
      contextRoot.addFilter(classOf[VersionEnforcerFilter], "/*", util.EnumSet.of(DispatcherType.REQUEST)).setAsyncSupported(true)
    }
  }

  private def setupRequestLocal(contextRoot: WebAppContext): Unit = {
    contextRoot.addFilter(classOf[RequestLocal], "*", util.EnumSet.of(DispatcherType.REQUEST)).setAsyncSupported(true)
  }

  private def enableResourcesCache(contextRoot: WebAppContext): Unit = {
    val holder = new FilterHolder(new ModifiedResourceFilter)
    holder.setInitParameter("startTimestamp", String.valueOf(System.currentTimeMillis))
    holder.setAsyncSupported(true)
    contextRoot.addFilter(holder, "/static/*", util.EnumSet.of(DispatcherType.REQUEST))
    contextRoot.addFilter(holder, "/ui-extensions/xlrelease-module.js", util.EnumSet.of(DispatcherType.REQUEST))
    contextRoot.addFilter(holder, "/ui-extensions/xlrelease-plugins.js", util.EnumSet.of(DispatcherType.REQUEST))
  }

  private def enableGzipHandler(contextRoot: WebAppContext): Unit = {
    val gzipHandler = new GzipHandler
    gzipHandler.setSyncFlush(true)
    gzipHandler.setIncludedMethods("GET", "POST", "PUT")
    gzipHandler.setMinGzipSize(xlrConfig.server_http_gzip_minSize.intValue)
    gzipHandler.setExcludedPaths(xlrConfig.server_http_gzip_excludedPaths.asScala.toSeq: _*)
    gzipHandler.setDeflaterPool(new DeflaterPool(CompressionPool.DEFAULT_CAPACITY, xlrConfig.server_http_gzip_compression, true))
    contextRoot.insertHandler(gzipHandler)
  }

  private def newSessionHandler = {
    val sessionHandler = new SessionHandler
    sessionHandler.setSessionTrackingModes(util.EnumSet.of(SessionTrackingMode.COOKIE))
    sessionHandler.setHttpOnly(true)
    if (!serverConfiguration.isSsl) {
      logger.debug(s"Setting up Secure Cookie Enabled to - ${serverConfiguration.isSecureCookieEnabled}")
      sessionHandler.getSessionCookieConfig.setSecure(serverConfiguration.isSecureCookieEnabled)
    }
    sessionHandler.setSameSite(SameSite.valueOf(xlrConfig.server_http_cookie_sameSite.getAttributeValue.toUpperCase))
    sessionHandler
  }

  private def enableInstrumentedHandler(contextRoot: WebAppContext): Unit = {
    val handler = new TimedHandler(meterRegistry, Tags.empty())
    contextRoot.insertHandler(handler)
  }

  private def setPropertyIfNull(key: String, value: String): Unit = {
    // skip null new values
    if (value != null) {
      if (System.getProperty(key) == null) {
        if (logger.isDebugEnabled) {
          logger.debug(s"Setting system property - $key")
        }
        System.setProperty(key, value)
      }
      else if (logger.isDebugEnabled) {
        logger.debug(s"Not overriding already specified system property - $key")
      }
    }
  }

  private def setupAccessControlFilter(contextRoot: WebAppContext): Unit = {
    val filterHolder = new FilterHolder(new XlrAccessControlFilter(permissionEnforcer, serviceManager, xlrConfig.maintenanceModeEnabled))
    contextRoot.addFilter(filterHolder, "/*", util.EnumSet.of(DispatcherType.REQUEST))
  }
}
